package uoc.edu.timestone.model;

import java.io.Serializable;

/**
 * This class represents an alert geerated by system
 */
public class Alert implements Serializable {

    public enum AlertType {
        ALERT_TYPE_ENTER,
        ALERT_TYPE_EXIT,
        ALERT_TYPE_ERROR
    }

    private String id;
    private String userId;
    private String centerId;
    private AlertType type;
    private String date;
    private String time;

    /**
     * Alert class empty constructor
     */
    public Alert() {
    }

    /**
     * Alert class parameter constructor
     *
     * @param id     Autogenerated database identifier
     * @param user   User which will be the alert target
     * @param center Center in which the alert was generated
     * @param type   Type of alert
     * @param date   Date of alert generation
     * @param time   Time of alert generation
     */
    public Alert(String id, User user, Center center, AlertType type, String date, String time) {
        this.id = id;
        this.userId = user.getId();
        this.centerId = center.getId();
        this.type = type;
        this.date = date;
        this.time = time;
    }

    public Alert(String id, String userId, String centerId, AlertType type, String date, String time) {
        this.id = id;
        this.userId = userId;
        this.centerId = centerId;
        this.type = type;
        this.date = date;
        this.time = time;
    }


    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getUserId() {
        return userId;
    }

    public void setUserId(String userId) {
        this.userId = userId;
    }

    public String getCenterId() {
        return centerId;
    }

    public void setCenterId(String centerId) {
        this.centerId = centerId;
    }

    public AlertType getType() {
        return type;
    }

    public void setType(AlertType type) {
        this.type = type;
    }

    public String getDate() {
        return date;
    }

    public void setDate(String date) {
        this.date = date;
    }

    public String getTime() {
        return time;
    }

    public void setTime(String time) {
        this.time = time;
    }
}
